#include <Arduino.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

// the following values are used for the temperature calculation
#define VCC 5.0                                                     // base voltage given by the Arduino
#define SERIES_RESISTOR 2000.0                                      // the resistor used for the voltage divider
#define THERMISTOR 100000.0                                         // the base resistance of your Thermistor (provided by the datasheet)
#define B 3950                                                      // b value of your Thermistor (provided by the datasheet)
#define NOMINAL 25                                                  // the temperature your thermistor reaches the base Resistance (provided by the datasheet)

// the following values are describing, which pins you (I) use
#define MOTOR 3                                                     // pin you used for the motor (make sure it'S a PWM)

#define HEAT_ZONE_1 26                                              // pin used to activate/deactivate your first heatzone
#define HEAT_ZONE_2 27                                              // pin used to activate/deactivate your second heatzone
#define HEAT_ZONE_3 28                                              // pin used to activate/deactivate your third heatzone

#define THERMISTOR_1 A0                                             // analog pin used to read the voltage->temperature of your first thermistor
#define THERMISTOR_2 A1                                             // analog pin used to read the voltage->temperature of your first thermistor
#define THERMISTOR_3 A2                                             // analog pin used to read the voltage->temperature of your first thermistor 

//#define BUTTON 29                                                    // pin used for detecting whether the button is pressed
//#define SWITCH_UP 30                                                 // pin used for detecting whether the switch is pressed up
//#define SWITCH_DOWN 31                                               // pin used for detecting whether the switch is pressed down
//#define POTI AX                                                     // analog pin used to read the poti value

//#define STATUS_LED_HEATZONE_1 X                                     // LED which indicates whether the desired temperature has been reached
//#define STATUS_LED_HEATZONE_2 X                                     // LED which indicates whether the desired temperature has been reached
//#define STATUS_LED_HEATZONE_3 X                                     // LED which indicates whether the desired temperature has been reached


//String material = "PLA";

class HeatZone {
  int pinHeater;
  int pinThermistor;
  int wTemp = 250;
  
public:
  // Constructor for heat zones with heater and thermistor pins
  HeatZone(int heaterPin, int thermistorPin) {
    pinHeater = heaterPin;
    pinThermistor = thermistorPin;
    pinMode(pinHeater, OUTPUT);
  }

  // function that regulates the Temperature
  void checkTemp() {
    Serial.print(getTemp());
    if (int(getTemp()) <= wTemp) {
      turnOn();
      Serial.println("  on");
    } else {
      turnOff();
      Serial.println("  off");
    }
  }

  // Function to get the temperature
  float getTemp() {
    float analogVoltage = analogRead(pinThermistor) * (VCC / 1023.0);
    //Serial.println(analogVoltage);

    float resistance = SERIES_RESISTOR * analogVoltage / (VCC - analogVoltage);
    //Serial.println(resistance);

    float steinhart = resistance / THERMISTOR;
    steinhart = log(steinhart) / B;
    steinhart += 1.0 / (NOMINAL + 273.15);
    steinhart = 1.0 / steinhart;
    steinhart -= 273.15;
    return steinhart;
  }

  // Function to set temperature
  void setTemp(int temp) {
    wTemp = temp;
  }

  int getWTemp() {
    return wTemp;
  }
    
  // Function to turn on the heater zone
  void turnOn() {
    digitalWrite(pinHeater, HIGH);
  }

  // Function to turn off the heater zone
  void turnOff() {
    digitalWrite(pinHeater, LOW);
  }

private:
  // 
};

class Motor {
  int pMotor;
  int spinRate = 100;
  
public:
  Motor(int pinMotor) {
    pMotor = pinMotor;
    pinMode(pMotor, OUTPUT);
  }

  void startMotor() {
    int value = spinRate*2.55;
    analogWrite(pMotor, value);
  }

  void stopMotor() {
    analogWrite(pMotor, LOW);
  }
};

// Define instances of HeatZone class for each heat zone
HeatZone heatZone1(HEAT_ZONE_1, THERMISTOR_1);
HeatZone heatZone2(HEAT_ZONE_2, THERMISTOR_2);
HeatZone heatZone3(HEAT_ZONE_3, THERMISTOR_3);

Motor motor(MOTOR);

void setup() {
  Serial.begin(9600);

  //heatZone1.setTemp(250);
  //heatZone2.setTemp(250);
  //heatZone3.setTemp(250);
}

void loop() {
  //motor.startMotor();

  //heatZone1.checkTemp();
  //heatZone2.checkTemp();
  //heatZone3.checkTemp();

  Serial.println();
  
  printAllTemperatures();

  delay(1000);
}

void printAllTemperatures() {
  Serial.println("Temperaturen der Heizzonen:");
  Serial.print("Heizzone 1: ");
  Serial.print(heatZone1.getTemp());
  Serial.println(" Grad Celsius");
  Serial.print("Heizzone 2: ");
  Serial.print(heatZone2.getTemp());
  Serial.println(" Grad Celsius");
  Serial.print("Heizzone 3: ");
  Serial.print(heatZone3.getTemp());
  Serial.println(" Grad Celsius");
  Serial.println();
}

